@@ -3,19 +3,18 @@ package App::BloomUtils;
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
 our $DATE = '2020-05-24'; # DATE
 our $DIST = 'App-BloomUtils'; # DIST
-our $VERSION = '0.003'; # VERSION
+our $VERSION = '0.004'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 use Log::ger;
 
+use POSIX qw(ceil);
+
 our %SPEC;
 
-$SPEC{gen_bloom_filter} = {
-    v => 1.1,
-    summary => 'Generate bloom filter',
-    description => <<'_',
+my $desc1 = <<'_';
 
 You supply lines of text from STDIN and it will output the bloom filter bits on
 STDOUT. You can also customize `num_bits` (`m`) and `num_hashes` (`k`), or, more
@@ -43,32 +42,47 @@ easily, `num_items` and `fp_rate`. Some rules of thumb to remember:
 
 Ref: https://corte.si/posts/code/bloom-filter-rules-of-thumb/index.html
 
+**FAQ**
+
+* Why does two different false positive rates (e.g. 1% and 0.1%) give the same bloom filter size?
+
+  The parameter `m` is rounded upwards to the nearest power of 2 (e.g. 1024*8
+  bits becomes 1024*8 bits but 1025*8 becomes 2048*8 bits), so sometimes two
+  false positive rates with different `m` get rounded to the same value of `m`.
+  Use the `bloom_filter_calculator` routine to see the `actual_m` and `actual_p`
+  (actual false-positive rate).
+
 _
+
+$SPEC{gen_bloom_filter} = {
+    v => 1.1,
+    summary => 'Generate bloom filter',
+    description => $desc1,
     args => {
         num_bits => {
             description => <<'_',
 
-The default is 80000 (generates a ~10KB bloom filter). If you supply 10,000 items
-(meaning 1 byte per 1 item) then the false positive rate will be ~2%. If you
-supply fewer items the false positive rate is smaller and if you supply more
-than 10,000 items the false positive rate will be higher.
+The default is 16384*8 bits (generates a ~16KB bloom filter). If you supply 16k
+items (meaning 1 byte per 1 item) then the false positive rate will be ~2%. If
+you supply fewer items the false positive rate is smaller and if you supply more
+than 16k items the false positive rate will be higher.
 
 _
-            schema => 'uint*',
-            #default => 8*10000,
+            schema => 'posint*',
+            #default => 8*16384,
             cmdline_aliases => {m=>{}},
         },
         num_hashes => {
-            schema => 'num*',
+            schema => 'posint*',
             cmdline_aliases => {k=>{}},
-            #default => 5.7,
+            #default => 6,
         },
         num_items => {
-            schema => 'uint*',
+            schema => 'posint*',
             cmdline_aliases => {n=>{}},
         },
         false_positive_rate => {
-            schema => 'percent*',
+            schema => ['float*', max=>0.5],
             cmdline_aliases => {
                 fp_rate => {},
                 p => {},
@@ -77,15 +91,11 @@ _
     },
     'cmdline.skip_format' => 1,
     args_rels => {
-        'choose_all&' => [
-            [qw/num_bits num_hashes/],
-            [qw/num_items false_positive_rate/],
-        ],
-        'choose_one' => [qw/num_bits num_items/],
     },
     examples => [
         {
-            summary => 'Create a bloom filter for 100k items and 0.1% false-positive rate',
+            summary => 'Create a bloom filter for 100k items and 0.1% maximum false-positive rate '.
+                '(actual bloom size and false-positive rate will be shown on stderr)',
             argv => [qw/--num-items 100000 --fp-rate 0.1%/],
             'x.doc.show_result' => 0,
             test => 0,
@@ -100,22 +110,28 @@ sub gen_bloom_filter {
 
     my %args = @_;
 
-    my ($m, $k);
-
+    my $res;
     if (defined $args{num_items}) {
-        my $res = bloom_filter_calculator(
+        $res = bloom_filter_calculator(
             num_items => $args{num_items},
+            num_bits => $args{num_bits},
+            num_hashes => $args{num_hashes},
             false_positive_rate => $args{false_positive_rate},
             num_hashes_to_bits_per_item_ratio => 0.7,
         );
-        return $res unless $res->[0] == 200;
-        $m = int($res->[2]{m});
-        $k = $res->[2]{k};
     } else {
-        $m = $args{num_bits} // 80_000;
-        $k = $args{num_hashes} // 5.7;
+        $res = bloom_filter_calculator(
+            num_bits => $args{num_bits} // 16384*8,
+            num_hashes => $args{num_hashes} // 6,
+
+            num_items => int($args{num_bits} / 8),
+        );
     }
-    log_trace "Will be creating bloom filter with m=%d, k=%.1f", $m, $k;
+    return $res unless $res->[0] == 200;
+    my $m = $res->[2]{actual_m};
+    my $k = $res->[2]{actual_k};
+    log_info "Will be creating bloom filter with num_bits (m)=%d, num_hashes (k)=%d, actual false-positive rate=%.5f%% (when num_items=%d), bloom filter size=%d bytes",
+        $m, $k, $res->[2]{actual_p}*100, $res->[2]{n}, $res->[2]{actual_bloom_size};
 
     my $bf = Algorithm::BloomFilter->new($m, $k);
     my $i = 0;
@@ -179,29 +195,7 @@ sub check_with_bloom_filter {
 $SPEC{bloom_filter_calculator} = {
     v => 1.1,
     summary => 'Help calculate num_bits (m) and num_hashes (k)',
-    description => <<'_',
-
-Bloom filter is setup using two parameters: `num_bits` (`m`) which is the size
-of the bloom filter (in bits) and `num_hashes` (`k`) which is the number of hash
-functions to use which will determine the write and lookup speed.
-
-Some rules of thumb:
-
-* One byte per item in the input set gives about a 2% false positive rate. So if
-  you expect two have 1024 elements, create a 1KB bloom filter with about 2%
-  false positive rate. For other false positive rates:
-
-    1%    -  9.6 bits per item
-    0.1%  - 14.4 bits per item
-    0.01% - 19.2 bits per item
-
-* Optimal number of hash functions is 0.7 times number of bits per item.
-
-* What is an acceptable false positive rate? This depends on your needs.
-
-Ref: https://corte.si/posts/code/bloom-filter-rules-of-thumb/index.html
-
-_
+    description => $desc1,
     args => {
         num_items => {
             summary => 'Expected number of items to add to bloom filter',
@@ -210,8 +204,13 @@ _
             req => 1,
             cmdline_aliases => {n=>{}},
         },
+        num_bits => {
+            summary => 'Number of bits to set for the bloom filter',
+            schema => 'posint*',
+            cmdline_aliases => {m=>{}},
+        },
         false_positive_rate => {
-            schema => 'num*',
+            schema => ['float*', max=>0.5],
             default => 0.02,
             cmdline_aliases => {
                 fp_rate => {},
@@ -219,49 +218,70 @@ _
             },
         },
         num_hashes => {
-            schema => 'num*',
+            schema => 'posint*',
             cmdline_aliases => {k=>{}},
         },
         num_hashes_to_bits_per_item_ratio => {
             summary => '0.7 (the default) is optimal',
             schema => 'num*',
-            default => 0.7,
         },
     },
     args_rels => {
-        choose_one => [qw/num_hashes num_hashes_to_bits_per_item_ratio/],
+        'choose_one&' => [
+            [qw/num_hashes num_hashes_to_bits_per_item_ratio/],
+        ],
     },
 };
 sub bloom_filter_calculator {
     my %args = @_;
 
+    my $num_hashes_to_bits_per_item_ratio = $args{num_hashes_to_bits_per_item_ratio};
+    $num_hashes_to_bits_per_item_ratio //= 0.7 unless defined($args{num_bits}) && defined($args{num_items});
+
     my $num_items = $args{num_items};
-    my $fp_rate   = $args{false_positive_rate};
+    my $fp_rate   = $args{false_positive_rate} // 0.02;
+    my $num_bits = $args{num_bits} // ($num_items * log(1/$fp_rate)/ log(2)**2);
 
-    my $num_bits = $num_items * log(1/$fp_rate)/ log(2)**2;
     my $num_bits_per_item = $num_bits / $num_items;
     my $num_hashes = $args{num_hashes} //
-        (defined $args{num_hashes_to_bits_per_item_ratio} ? $args{num_hashes_to_bits_per_item_ratio}*$num_bits_per_item : undef) //
+        (defined $num_hashes_to_bits_per_item_ratio ? $num_hashes_to_bits_per_item_ratio*$num_bits_per_item : undef) //
         ($num_bits / $num_items * log(2));
-    my $num_hashes_to_bits_per_item_ratio = $args{num_hashes_to_bits_per_item_ratio} //
-        $num_hashes / $num_bits_per_item;
+    $num_hashes_to_bits_per_item_ratio //= $num_hashes / $num_bits_per_item;
+
+    my $actual_num_hashes = ceil($num_hashes);
+    my $num_bits_2power = sprintf "%.6f", (log($num_bits) / log(2));
+    my $actual_num_bits = 2**ceil($num_bits_2power);
+    my $actual_fp_rate = (1 - exp(-$actual_num_hashes*$num_items/$actual_num_bits))**$actual_num_hashes;
+    my $actual_bloom_size = ($actual_num_bits/8) + 3;
 
     [200, "OK", {
         num_bits   => $num_bits,
         m          => $num_bits,
+
         num_items  => $num_items,
         n          => $num_items,
+
         num_hashes => $num_hashes,
-        num_hashes_to_bits_per_item_ratio => $num_hashes_to_bits_per_item_ratio,
         k          => $num_hashes,
+
+        num_hashes_to_bits_per_item_ratio => $num_hashes_to_bits_per_item_ratio,
+
         fp_rate    => $fp_rate,
         p          => $fp_rate,
+
         num_bits_per_item => $num_bits / $num_items,
         'm/n'             => $num_bits / $num_items,
+
+        actual_num_bits   => $actual_num_bits,
+        actual_m          => $actual_num_bits,
+        actual_num_hashes => ceil($num_hashes),
+        actual_k          => ceil($num_hashes),
+        actual_fp_rate    => $actual_fp_rate,
+        actual_p          => $actual_fp_rate,
+        actual_bloom_size => $actual_bloom_size,
     }];
 }
 
-
 1;
 # ABSTRACT: Utilities related to bloom filters
 
@@ -277,7 +297,7 @@ App::BloomUtils - Utilities related to bloom filters
 
 =head1 VERSION
 
-This document describes version 0.003 of App::BloomUtils (from Perl distribution App-BloomUtils), released on 2020-05-24.
+This document describes version 0.004 of App::BloomUtils (from Perl distribution App-BloomUtils), released on 2020-05-24.
 
 =head1 DESCRIPTION
 
@@ -287,6 +307,12 @@ This distributions provides the following command-line utilities:
 
 =item * L<bloom-filter-calculator>
 
+=item * L<bloomcalc>
+
+=item * L<bloomchk>
+
+=item * L<bloomgen>
+
 =item * L<check-with-bloom-filter>
 
 =item * L<gen-bloom-filter>
@@ -304,11 +330,9 @@ Usage:
 
 Help calculate num_bits (m) and num_hashes (k).
 
-Bloom filter is setup using two parameters: C<num_bits> (C<m>) which is the size
-of the bloom filter (in bits) and C<num_hashes> (C<k>) which is the number of hash
-functions to use which will determine the write and lookup speed.
-
-Some rules of thumb:
+You supply lines of text from STDIN and it will output the bloom filter bits on
+STDOUT. You can also customize C<num_bits> (C<m>) and C<num_hashes> (C<k>), or, more
+easily, C<num_items> and C<fp_rate>. Some rules of thumb to remember:
 
 =over
 
@@ -316,29 +340,55 @@ Some rules of thumb:
 you expect two have 1024 elements, create a 1KB bloom filter with about 2%
 false positive rate. For other false positive rates:
 
-1%    -  9.6 bits per item
-0.1%  - 14.4 bits per item
-0.01% - 19.2 bits per item
+10%    -  4.8 bits per item
+ 1%    -  9.6 bits per item
+ 0.1%  - 14.4 bits per item
+ 0.01% - 19.2 bits per item
 
-=item * Optimal number of hash functions is 0.7 times number of bits per item.
+=item * Optimal number of hash functions is 0.7 times number of bits per item. Note
+that the number of hashes dominate performance. If you want higher
+performance, pick a smaller number of hashes. But for most cases, use the the
+optimal number of hash functions.
 
-=item * What is an acceptable false positive rate? This depends on your needs.
+=item * What is an acceptable false positive rate? This depends on your needs. 1% (1
+in 100) or 0.1% (1 in 1,000) is a good start. If you want to make sure that
+user's chosen password is not in a known wordlist, a higher false positive
+rates will annoy your user more by rejecting her password more often, while
+lower false positive rates will require a higher memory usage.
 
 =back
 
 Ref: https://corte.si/posts/code/bloom-filter-rules-of-thumb/index.html
 
+B<FAQ>
+
+=over
+
+=item * Why does two different false positive rates (e.g. 1% and 0.1%) give the same bloom filter size?
+
+The parameter C<m> is rounded upwards to the nearest power of 2 (e.g. 1024*8
+bits becomes 1024*8 bits but 1025*8 becomes 2048*8 bits), so sometimes two
+false positive rates with different C<m> get rounded to the same value of C<m>.
+Use the C<bloom_filter_calculator> routine to see the C<actual_m> and C<actual_p>
+(actual false-positive rate).
+
+=back
+
 This function is not exported.
 
 Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<false_positive_rate> => I<num> (default: 0.02)
+=item * B<false_positive_rate> => I<float> (default: 0.02)
 
-=item * B<num_hashes> => I<num>
+=item * B<num_bits> => I<posint>
 
-=item * B<num_hashes_to_bits_per_item_ratio> => I<num> (default: 0.7)
+Number of bits to set for the bloom filter.
+
+=item * B<num_hashes> => I<posint>
+
+=item * B<num_hashes_to_bits_per_item_ratio> => I<num>
 
 0.7 (the default) is optimal.
 
@@ -413,7 +463,7 @@ Examples:
 
 =over
 
-=item * Create a bloom filter for 100k items and 0.1% false-positive rate:
+=item * Create a bloom filter for 100k items and 0.1% maximum false-positive rate (actual bloom size and false-positive rate will be shown on stderr):
 
  gen_bloom_filter( false_positive_rate => "0.1%", num_items => 100000);
 
@@ -449,24 +499,38 @@ lower false positive rates will require a higher memory usage.
 
 Ref: https://corte.si/posts/code/bloom-filter-rules-of-thumb/index.html
 
+B<FAQ>
+
+=over
+
+=item * Why does two different false positive rates (e.g. 1% and 0.1%) give the same bloom filter size?
+
+The parameter C<m> is rounded upwards to the nearest power of 2 (e.g. 1024*8
+bits becomes 1024*8 bits but 1025*8 becomes 2048*8 bits), so sometimes two
+false positive rates with different C<m> get rounded to the same value of C<m>.
+Use the C<bloom_filter_calculator> routine to see the C<actual_m> and C<actual_p>
+(actual false-positive rate).
+
+=back
+
 This function is not exported.
 
 Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<false_positive_rate> => I<percent>
+=item * B<false_positive_rate> => I<float>
 
-=item * B<num_bits> => I<uint>
+=item * B<num_bits> => I<posint>
 
-The default is 80000 (generates a ~10KB bloom filter). If you supply 10,000 items
-(meaning 1 byte per 1 item) then the false positive rate will be ~2%. If you
-supply fewer items the false positive rate is smaller and if you supply more
-than 10,000 items the false positive rate will be higher.
+The default is 16384*8 bits (generates a ~16KB bloom filter). If you supply 16k
+items (meaning 1 byte per 1 item) then the false positive rate will be ~2%. If
+you supply fewer items the false positive rate is smaller and if you supply more
+than 16k items the false positive rate will be higher.
 
-=item * B<num_hashes> => I<num>
+=item * B<num_hashes> => I<posint>
 
-=item * B<num_items> => I<uint>
+=item * B<num_items> => I<posint>
 
 
 =back
